package no.valg.valgnatt.database.hibernate;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import no.valg.valgnatt.database.Konstanter;
import no.valg.valgnatt.database.domene.ValgType;
import no.valg.valgnatt.database.domene.ValideringStatus;
import org.hibernate.annotations.CreationTimestamp;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import java.util.Date;

@Entity
@Table(name = "stemmeskjema")
@NoArgsConstructor //Hibernate-krav
@AllArgsConstructor //Builder-krav
@Builder
public class Stemmeskjema implements java.io.Serializable {

	@Id
	@Getter
	@Setter
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "stemmeskjema_id_generator")
	@SequenceGenerator(name = "stemmeskjema_id_generator", sequenceName = "stemmeskjema_id_sequence")
	@Column(name = "id", unique = true, nullable = false)
	private Integer id;

	@Getter
	@Setter
	@Column(name = "data")
	private String data;

	@Column(name = "valideringstatus")
	private String valideringstatus;

	@Getter
	@Setter
	@Column(name = "validering_info", length = 4000)
	private String valideringInfo;

	@Column(name = "valgtype")
	private String valgType;

	@Getter
	@Setter
	@Column(name = "opptaltstatus", nullable = false)
	private Integer opptaltstatus;

	@Getter
	@Setter
	@Column(name = "kommnr", length = 16)
	private String kommnr;

	@Getter
	@Setter
	@Column(name = "kretsnr", length = 16)
	private String kretsnr;

	@Getter
	@Setter
	@Column(name = "skjematype", length = 100)
	private String skjemaType;

	@Getter
	@Column(name = "opprettet_tid", updatable = false)
	@CreationTimestamp
	private Date opprettetTid;

	@Getter
	@Setter
	@Column(name = "original_tid")
	private Date originalTid;

	public ValideringStatus getValideringstatus() {
		return ValideringStatus.valueOf(this.valideringstatus);
	}


	public void setValideringstatus(ValideringStatus valideringstatus) {
		this.valideringstatus = valideringstatus.name();
	}

	public boolean erKommunestyrevalgForOslo() {
		return getValgType() == ValgType.KO && Konstanter.OSLO_KNR.equals(kommnr);
	}

	public ValgType getValgType() {
		return ValgType.valueOf(this.valgType);
	}

	public void setValgType(ValgType valgType) {
		this.valgType = valgType.name();
	}
}
